import {
  select,
  csv,
  json,
  scaleLinear,
  scaleSqrt,
  extent,
  max,
  axisLeft,
  axisBottom
} from 'd3'

import { scatterplot } from './scatterplot'
import { tidyStr, getAxisValue } from './helpers'

const svg = select('#svg-visu')
const controlsContainer = select('#controls')

const margin = { top: 60, right: 160, bottom: 72, left: 100 }
const width = +svg.attr('width')
const height = +svg.attr('height')

// state
let usedData
let allShips = false

const dataColumns = [
  'length',
  'crew',
  'passengers',
  'cargo_capacity',
  'cost_in_credits'
]

const columns = {
  y: 'cost_in_credits',
  x: 'length',
  z: 'crew'
}

const controls = (selection, props) => {
  const { 
    dataColumns,
    columns
  } = props
  
  const row1 = controlsContainer.select('#row1')
  const row2 = controlsContainer.select('#row2')
 
  Object.keys(columns).forEach(column => {
    const container = row1.selectAll('.col-' + column).data([null])
    const contEnter = container.enter().append('div')
    	.attr('class', 'ctrl-g col-' + column)
    contEnter.append('div').attr('class', 'col-label')
    	.text('Column ' + column.toUpperCase())
    contEnter.merge(container)
	    .selectAll('.column-key').data(dataColumns)
  	  	.enter().append('div')
    			.attr('class', 'column-key')
    		.merge(container.selectAll('.column-key'))
	    		.text(d => d)
  	    	.attr('class', d => {
    	    	const extent = columns[column] === d
      	  		? ' selected'
        			: ''
	      		return 'column-key' + extent
  				})
    	    .on('click', d => {
      	    columns[column] = d
    				render();
        	})
  })
  
  const filterContainer = row1.selectAll('.filter-container').data([null])
  const filterEnter = filterContainer.enter().append('div')
    .attr('class', 'filter-container')
  filterEnter.append('p')
  	.text('Showing ')
  	.append('span')
  		.attr('id', 'max-display')
  		.text(() => allShips
           ? 'all ships'
           : 'small ships'
           )
  filterEnter.append('input')
  	.attr('class', 'checkbox')
  	.attr('type', 'checkbox')
  	.attr('id', 'all-ships-toggle')
  	.on('change', function() {
    	allShips = this.checked
    	render()
  	})
  filterEnter.append('label')
  	.attr('for', 'all-ships-toggle')
  	.text('Show all ships')
  		
}

const render = () => {
  
  controls(svg, {
    dataColumns,
    columns
  })
  
  console.log('allShips:',allShips)
  
  const data = allShips
  	? usedData
  	: usedData.filter(d => d.cost_in_credits < 1000000)
  
  scatterplot(svg, {
    data: data,
  	title: 'Dynamic starships comparison',
  	xValue: getAxisValue(columns.x),
  	yValue: getAxisValue(columns.y),
  	zValue: getAxisValue(columns.z),
  	xAxisLabel: 'Diameter',
  	yAxisLabel: 'Population',
  	zAxisLabel: 'Diameter',
    columns,
    margin,
    width,
    height,
    tidyStr
  })

}

json('starships.json')
  .then(data => {
  	console.log(data[0])
  	const filtered = data.filter(d => {
      let faults = dataColumns.filter(key => isNaN(d[key]))
      return faults.length === 0
    })
    filtered.forEach(d => {
      dataColumns.forEach(key => d[key] = +d[key])
    });
    usedData = filtered
    
    render();
	})
